@@ -1,6 +1,6 @@
 //===========================================================================
 // @(#) $Name:$
-// @(#) $Id: mcblockc.cc 9417 2017-06-04 03:07:40Z dwm $
+// @(#) $Id: mcblockc.cc 9691 2017-06-30 09:45:02Z dwm $
 //===========================================================================
 //  Copyright (c) Daniel W. McRobb 2017
 //  All rights reserved.
@@ -68,7 +68,7 @@ extern "C" {
 #include "DwmMcBlockRequestMessage.hh"
 #include "DwmMcBlockResponseMessage.hh"
 
-static const Dwm::SvnTag svntag("@(#) $DwmPath: dwm/mcplex/mcblock/tags/mcblock-0.1.0/apps/mcblockc/mcblockc.cc 9417 $");
+static const Dwm::SvnTag svntag("@(#) $DwmPath: dwm/mcplex/mcblock/tags/mcblock-0.1.1/apps/mcblockc/mcblockc.cc 9691 $");
 
 using namespace std;
 using namespace Dwm;
@@ -173,12 +173,14 @@ bool Connect(const string & host, Socket & s)
 //----------------------------------------------------------------------------
 static void Usage(const string & argv0)
 {
-  cerr << "Usage: " << argv0 << " activate tableName prefix(es)\n"
-       << "       " << argv0 << " deactivate tableName prefix(es)\n"
-       << "       " << argv0 << " getactive tableName\n"
-       << "       " << argv0 << " getaddrules tableName\n"
-       << "       " << argv0 << " loghit tableName ipv4addr\n"
-       << "       " << argv0 << " search ipv4addr\n";
+  cerr << "Usage: " << argv0 << " [-h host] activate tableName prefix(es)\n"
+       << "       " << argv0 << " [-h host] deactivate tableName prefix(es)\n"
+       << "       " << argv0 << " [-h host] getactive tableName\n"
+       << "       " << argv0 << " [-h host] getaddrules tableName\n"
+       << "       " << argv0 << " [-h host] loghit tableName ipv4addr\n"
+       << "       " << argv0 << " [-h host] search ipv4addr\n"
+       << "       " << argv0 << " [-h host] get tableName ipv4addr\n"
+       << "       " << argv0 << " [-h host] edit tableName ipv4addr\n";
   return;
 }
 
@@ -359,6 +361,99 @@ static bool LogHit(Socket & s, const string & secret,
   return rc;
 }
 
+//----------------------------------------------------------------------------
+//!  
+//----------------------------------------------------------------------------
+static bool Get(Socket & s, const string & secret,
+                const string & table, const string & addrstr,
+                Json::Value & jv)
+{
+  bool  rc = false;
+  Ipv4Address  addr(addrstr);
+  McBlock::ResponseMessage  resp;
+  McBlock::RequestMessage   req((McBlock::GetReq(table, addr)));
+  if (RequestResponse(s, secret, req, resp)) {
+    if (! resp.Json().empty()) {
+      jv = resp.Json();
+      rc = true;
+    }
+  }
+  return rc;
+}
+
+//----------------------------------------------------------------------------
+//!  
+//----------------------------------------------------------------------------
+static bool Edit(Socket & s, const string & secret,
+                const string & table, const string & addrstr)
+{
+  bool  rc = false;
+  Json::Value  jv;
+  if (Get(s, secret, table, addrstr, jv)) {
+    cout << "Editing prefix " << jv["prefix"].asString() << " in table "
+         << jv["table"].asString() << endl;
+    string  st;
+    string  countryCode;
+    while (countryCode.empty()) {
+      cout << "countryCode [" << jv["countryCode"].asString() << "]: "
+           << std::flush;
+      std::getline(cin, st);
+      if (! st.empty()) {
+        if (st.length() == 2) {
+          countryCode = st;
+        }
+      }
+      else {
+        countryCode = jv["countryCode"].asString();
+      }
+    }
+    jv["countryCode"] = countryCode;
+  readDaysRemaining:
+    cout << "daysRemaining [" << jv["daysRemaining"].asUInt() << "]: "
+         << std::flush;
+    std::getline(cin, st);
+    if (! st.empty()) {
+      try {
+        jv["daysRemaining"] = static_cast<Json::UInt>(stoul(st, nullptr, 10));
+      }
+      catch (...) {
+        goto readDaysRemaining;
+      }
+    }
+
+    jv["type"] = "edit";
+    McBlock::ResponseMessage  resp;
+    McBlock::RequestMessage   req(jv);
+    if (RequestResponse(s, secret, req, resp)) {
+      if (! resp.Json().empty()) {
+        cout << resp.Json().toStyledString() << '\n';
+        rc = true;
+      }
+    }
+  }
+  return rc;
+}
+
+
+//----------------------------------------------------------------------------
+//!  
+//----------------------------------------------------------------------------
+static bool Get(Socket & s, const string & secret,
+                const string & table, const string & addrstr)
+{
+  bool  rc = false;
+  Ipv4Address  addr(addrstr);
+  McBlock::ResponseMessage  resp;
+  McBlock::RequestMessage   req((McBlock::GetReq(table, addr)));
+  if (RequestResponse(s, secret, req, resp)) {
+    if (! resp.Json().empty()) {
+      cout << resp.Json().toStyledString() << '\n';
+      rc = true;
+    }
+  }
+  return rc;
+}
+
 //----------------------------------------------------------------------------
 //!  
 //----------------------------------------------------------------------------
@@ -506,6 +601,44 @@ int main(int argc, char *argv[])
       exit(1);
     }
   }
+  else if (string(argv[nextArg]) == "get") {
+    if (argc > ++nextArg) {
+      string  table = argv[nextArg];
+      if (argc > ++nextArg) {
+        string  hostaddr = argv[nextArg];
+        if (Get(s, secret, table, hostaddr)) {
+          rc = 0;
+        }
+      }
+      else {
+        Usage(argv[0]);
+        exit(1);
+      }
+    }
+    else {
+      Usage(argv[0]);
+      exit(1);
+    }
+  }
+  else if (string(argv[nextArg]) == "edit") {
+    if (argc > ++nextArg) {
+      string  table = argv[nextArg];
+      if (argc > ++nextArg) {
+        string  hostaddr = argv[nextArg];
+        if (Edit(s, secret, table, hostaddr)) {
+          rc = 0;
+        }
+      }
+      else {
+        Usage(argv[0]);
+        exit(1);
+      }
+    }
+    else {
+      Usage(argv[0]);
+      exit(1);
+    }
+  }
   
   else {
     Usage(argv[0]);
