@@ -1,6 +1,6 @@
 //===========================================================================
 // @(#) $Name:$
-// @(#) $Id: DwmMcBlockResponder.cc 9443 2017-06-06 00:26:17Z dwm $
+// @(#) $Id: DwmMcBlockResponder.cc 9688 2017-06-30 09:25:29Z dwm $
 //===========================================================================
 //  Copyright (c) Daniel W. McRobb 2017
 //  All rights reserved.
@@ -62,7 +62,7 @@ extern "C" {
 #include "DwmMcBlockServer.hh"
 #include "DwmMcBlockTcpDrop.hh"
 
-static const Dwm::SvnTag  svntag("$DwmPath: dwm/mcplex/mcblock/tags/mcblock-0.1.0/apps/mcblockd/DwmMcBlockResponder.cc 9443 $");
+static const Dwm::SvnTag  svntag("$DwmPath: dwm/mcplex/mcblock/tags/mcblock-0.1.1/apps/mcblockd/DwmMcBlockResponder.cc 9688 $");
 
 using namespace std;
 
@@ -107,6 +107,39 @@ namespace Dwm {
       return rc;
     }
 
+    //------------------------------------------------------------------------
+    //!  
+    //------------------------------------------------------------------------
+    bool Responder::IsValidGetRequest(const Json::Value & request)
+    {
+      return (request.isMember("type")
+              && (request["type"].isString())
+              && (request["type"] == "get")
+              && (request.isMember("table"))
+              && (request["table"].isString())
+              && (request.isMember("ipv4addr"))
+              && (request["ipv4addr"].isString()));
+    }
+
+    //------------------------------------------------------------------------
+    //!  
+    //------------------------------------------------------------------------
+    bool Responder::IsValidEditRequest(const Json::Value & request)
+    {
+      return (request.isMember("type")
+              && request["type"].isString()
+              && (request["type"] == "edit")
+              && request.isMember("table")
+              && request["table"].isString()
+              && request.isMember("prefix")
+              && request["prefix"].isString()
+              && request.isMember("countryCode")
+              && request["countryCode"].isString()
+              && (request["countryCode"].asString().size() == 2)
+              && request.isMember("daysRemaining")
+              && request["daysRemaining"].isUInt());
+    }
+    
     //------------------------------------------------------------------------
     //!  
     //------------------------------------------------------------------------
@@ -328,6 +361,63 @@ namespace Dwm {
       return rc;
     }
 
+    //------------------------------------------------------------------------
+    //!  
+    //------------------------------------------------------------------------
+    Json::Value Responder::HandleGetRequest(const Json::Value & request)
+    {
+      Json::Value  rc;
+      Ipv4Address  addr(request["ipv4addr"].asString());
+      auto dbi = _server.GetDatabases().find(request["table"].asString());
+      if (dbi != _server.GetDatabases().end()) {
+        pair<Ipv4Prefix,DbEntry>  match;
+        if (dbi->second.FindLongest(addr, match)) {
+          rc["table"] = dbi->first;
+          rc["prefix"] = match.second.Prefix().ToShortString();
+          rc["countryCode"] = match.second.Country();
+          TimeValue  now(true);
+          TimeValue  timeLeft(match.second.Interval().End());
+          timeLeft -= now;
+          uint32_t  daysLeft = (timeLeft.Secs() / (24 * 60 * 60));
+          rc["daysRemaining"] = daysLeft;
+        }
+      }
+      return rc;
+    }
+
+    //------------------------------------------------------------------------
+    //!  
+    //------------------------------------------------------------------------
+    Json::Value Responder::HandleEditRequest(const Json::Value & request)
+    {
+      Json::Value  rc;
+      Ipv4Prefix   pfx(request["prefix"].asString());
+      auto dbi = _server.GetDatabases().find(request["table"].asString());
+      if (dbi != _server.GetDatabases().end()) {
+        DbEntry  match;
+        if (dbi->second.Find(pfx, match)) {
+          TimeValue  now(true);
+          TimeValue  endTime(now);
+          endTime.Set(endTime.Secs()
+                      + (request["daysRemaining"].asUInt() * 24 * 60 * 60), 0);
+          TimeInterval  interval(match.Interval().Start(), endTime);
+          match.Interval(interval);
+          match.Country(request["countryCode"].asString());
+          dbi->second[pfx] = match;
+          dbi->second.Save("");
+          rc["edited"] = true;
+          rc["prefix"] = pfx.ToShortString();
+          rc["table"] = dbi->first;
+          rc["countryCode"] = match.Country();
+          TimeValue  timeLeft(match.Interval().End());
+          timeLeft -= now;
+          uint32_t  daysLeft = (timeLeft.Secs() / (24 * 60 * 60));
+          rc["daysRemaining"] = daysLeft;
+        }
+      }
+      return rc;
+    }
+    
     //------------------------------------------------------------------------
     //!  
     //------------------------------------------------------------------------
@@ -589,6 +679,12 @@ namespace Dwm {
       else if (IsValidLogHitRequest(request)) {
         return HandleLogHitRequest(request);
       }
+      else if (IsValidGetRequest(request)) {
+        return HandleGetRequest(request);
+      }
+      else if (IsValidEditRequest(request)) {
+        return HandleEditRequest(request);
+      }
       else {
         Syslog(LOG_ERR, "Invalid request");
         return Json::Value();
